//
//  File.swift
//  
//
//  Created by Yusuf Tör on 20/01/2023.
//
// swiftlint:disable all

import XCTest
@testable import SuperwallKit
import WebKit

final class RawWebMessageHandlerTests: XCTestCase {
  class MockWKScriptMessage: WKScriptMessage {
    private var internalBody: Any

    init(body: String) {
      self.internalBody = body
    }

    override var body: Any {
      return internalBody
    }
  }

  class MockWebEventDelegate: WebEventDelegate {
    var handledMessages: [PaywallMessage] = []

    func handle(_ message: PaywallMessage) {
      handledMessages.append(message)
    }
  }

  @MainActor
  func test_userContentController() {
    let delegate = MockWebEventDelegate()
    let rawWebMessageHandler = RawWebMessageHandler(delegate: delegate)
    let body = """
{\"direction\":\"from_paywall\",\"version\":1,\"payload\":{\"events\":[{\"event_name\":\"ping\",\"substitutions\":[{\"key\":\"open-url-1\",\"value\":\"Privacy Policy\",\"tagName\":\"a\",\"subType\":\"var\",\"skipInnerHTML\":false,\"properties\":[{\"prefix\":\"default\",\"property\":{\"clickBehavior\":{\"type\":\"open-url\",\"url\":\"https://moonsetlabs.com/fitness-ai/privacy_policy.html\"},\"type\":\"click-behavior\"}}]},{\"key\":\"open-url-2\",\"value\":\"Terms of Use\",\"tagName\":\"a\",\"subType\":\"var\",\"skipInnerHTML\":false,\"properties\":[{\"prefix\":\"default\",\"property\":{\"clickBehavior\":{\"type\":\"open-url\",\"url\":\"https://moonsetlabs.com/fitness-ai/terms_and_conditions.html\"},\"type\":\"click-behavior\"}}]},{\"key\":\"close-1\",\"value\":\"EXIT\",\"tagName\":\"a\",\"subType\":\"var\",\"skipInnerHTML\":false,\"properties\":[{\"prefix\":\"default\",\"property\":{\"clickBehavior\":{\"type\":\"close\"},\"type\":\"click-behavior\"}}]},{\"key\":\"restore-1\",\"value\":\"<div data-pw-var=\\\"Already a member\\\" class=\\\"text-block-21\\\" data-pw-found=\\\"true\\\">Already a member?</div><div data-pw-var=\\\"Restore\\\" class=\\\"text-block-20\\\" data-pw-found=\\\"true\\\">Restore</div>\",\"tagName\":\"div\",\"subType\":\"var\",\"skipInnerHTML\":false,\"properties\":[{\"prefix\":\"default\",\"property\":{\"clickBehavior\":{\"type\":\"restore\"},\"type\":\"click-behavior\"}}]},{\"key\":\"Help\",\"value\":\"HELP\",\"tagName\":\"a\",\"subType\":\"var\",\"skipInnerHTML\":false,\"properties\":[]},{\"key\":\"title\",\"value\":\"How your 7-day<br>FREE&nbsp;trial works\",\"tagName\":\"h1\",\"subType\":\"var\",\"skipInnerHTML\":false,\"properties\":[]},{\"key\":\"subtitle\",\"value\":\"Only $1.73 per week billed annually. That\'s 50-100x cheaper than a trainer.<br>\",\"tagName\":\"div\",\"subType\":\"var\",\"skipInnerHTML\":false,\"properties\":[]},{\"key\":\"bullet-1\",\"value\":\"Optimized workouts everyday<br>\",\"tagName\":\"div\",\"subType\":\"var\",\"skipInnerHTML\":false,\"properties\":[]},{\"key\":\"bullet-2\",\"value\":\"Adapted to your strength<br>\",\"tagName\":\"div\",\"subType\":\"var\",\"skipInnerHTML\":false,\"properties\":[]},{\"key\":\"bullet-3\",\"value\":\"347+ detailed exercise guides<br>\",\"tagName\":\"div\",\"subType\":\"var\",\"skipInnerHTML\":false,\"properties\":[]},{\"key\":\"bullet-4\",\"value\":\"World renowned coaches<br>\",\"tagName\":\"div\",\"subType\":\"var\",\"skipInnerHTML\":false,\"properties\":[]},{\"key\":\"bullet-5\",\"value\":\"Life changing advice<br>\",\"tagName\":\"div\",\"subType\":\"var\",\"skipInnerHTML\":false,\"properties\":[]},{\"key\":\"bullet-6\",\"value\":\"Over 1,000,000 happy users<br>\",\"tagName\":\"div\",\"subType\":\"var\",\"skipInnerHTML\":false,\"properties\":[]},{\"key\":\"timeline title\",\"value\":\"So how does my free trial work?\",\"tagName\":\"h1\",\"subType\":\"var\",\"skipInnerHTML\":false,\"properties\":[]},{\"key\":\"Timeline Row 1 Title\",\"value\":\"Today\",\"tagName\":\"h2\",\"subType\":\"var\",\"skipInnerHTML\":false,\"properties\":[]},{\"key\":\"Timeline Row 1 Subtitle\",\"value\":\"FitnessAI builds you daily workouts with optimized sets, reps and weights.<br>\",\"tagName\":\"div\",\"subType\":\"var\",\"skipInnerHTML\":false,\"properties\":[]},{\"key\":\"Timeline Row 2 Title\",\"value\":\"Tomorrow\",\"tagName\":\"h2\",\"subType\":\"var\",\"skipInnerHTML\":false,\"properties\":[]},{\"key\":\"Timeline Row 2 Subtitle\",\"value\":\"Domenic (your human coach) will message you to see how you\'re doing.<br>\",\"tagName\":\"div\",\"subType\":\"var\",\"skipInnerHTML\":false,\"properties\":[]},{\"key\":\"Timeline Row 3 Title\",\"value\":\"In 6 days &nbsp;⏰\",\"tagName\":\"h2\",\"subType\":\"var\",\"skipInnerHTML\":false,\"properties\":[]},{\"key\":\"Timeline Row 3 Subtitle\",\"value\":\"We\'ll remind you 24 hrs before charging your card.<br>\",\"tagName\":\"div\",\"subType\":\"var\",\"skipInnerHTML\":false,\"properties\":[]},{\"key\":\"callout-badge\",\"value\":\"40% OFF\",\"tagName\":\"div\",\"subType\":\"var\",\"skipInnerHTML\":false,\"properties\":[]},{\"key\":\"callout title\",\"value\":\"Just $1.73 per week\",\"tagName\":\"h1\",\"subType\":\"var\",\"skipInnerHTML\":false,\"properties\":[]},{\"key\":\"callout subtitle\",\"value\":\"7 days free, cancel anytime<br>\",\"tagName\":\"div\",\"subType\":\"var\",\"skipInnerHTML\":false,\"properties\":[]},{\"key\":\"Heading 26\",\"value\":\"A.I. powered workouts &amp; real human trainers means 1000s of success stories.\",\"tagName\":\"h1\",\"subType\":\"var\",\"skipInnerHTML\":false,\"properties\":[]},{\"key\":\"Privacy Policy\",\"value\":\"Please read our <a href=\\\"#\\\" data-pw-open-url=\\\"https://moonsetlabs.com/fitness-ai/privacy_policy.html\\\" class=\\\"link-5\\\" data-pw-found=\\\"true\\\" id=\\\"swElement545223572663353600\\\">Privacy Policy</a> and <a href=\\\"#\\\" data-pw-open-url=\\\"https://moonsetlabs.com/fitness-ai/terms_and_conditions.html\\\" class=\\\"link-6\\\" data-pw-found=\\\"true\\\" id=\\\"swElement545223571186568500\\\">Terms of Use</a> before joining. <br>\",\"tagName\":\"div\",\"subType\":\"var\",\"skipInnerHTML\":false,\"properties\":[]},{\"key\":\"Get in touch\",\"value\":\"Have a question? Get in touch!\",\"tagName\":\"div\",\"subType\":\"var\",\"skipInnerHTML\":false,\"properties\":[]},{\"key\":\"Already a member\",\"value\":\"Already a member?\",\"tagName\":\"div\",\"subType\":\"var\",\"skipInnerHTML\":false,\"properties\":[]},{\"key\":\"Restore\",\"value\":\"Restore\",\"tagName\":\"div\",\"subType\":\"var\",\"skipInnerHTML\":false,\"properties\":[]},{\"key\":\"purchase button subtitle\",\"value\":\"7 days free then only $89.99 per yr<br>\",\"tagName\":\"div\",\"subType\":\"var\",\"skipInnerHTML\":false,\"properties\":[]},{\"key\":\"purchase-primary\",\"value\":\"START&nbsp;7-DAY&nbsp;FREE&nbsp;TRIAL\",\"tagName\":\"a\",\"subType\":\"var\",\"skipInnerHTML\":false,\"properties\":[{\"prefix\":\"default\",\"property\":{\"type\":\"click-behavior\",\"clickBehavior\":{\"type\":\"purchase\",\"product\":\"primary\"}}}]},{\"key\":\"custom-1\",\"value\":\"Message Us\",\"tagName\":\"div\",\"subType\":\"var\",\"skipInnerHTML\":false,\"properties\":[{\"prefix\":\"default\",\"property\":{\"clickBehavior\":{\"type\":\"custom\",\"data\":\"intercom\"},\"type\":\"click-behavior\"}}]}],\"coordinates\":{\"open-url-1\":{\"x\":154.3779296875,\"y\":2450.09375,\"width\":103.02084350585938,\"height\":19,\"top\":2450.09375,\"right\":257.3987731933594,\"bottom\":2469.09375,\"left\":154.3779296875},\"open-url-2\":{\"x\":125.27417755126953,\"y\":2450.09375,\"width\":231.95782470703125,\"height\":41,\"top\":2450.09375,\"right\":357.2320022583008,\"bottom\":2491.09375,\"left\":125.27417755126953},\"close-1\":{\"x\":0,\"y\":46,\"width\":57.8125,\"height\":58,\"top\":46,\"right\":57.8125,\"bottom\":104,\"left\":0},\"restore-1\":{\"x\":15,\"y\":2658.09375,\"width\":363,\"height\":60,\"top\":2658.09375,\"right\":378,\"bottom\":2718.09375,\"left\":15},\"Help\":{\"x\":304.125,\"y\":46,\"width\":88.875,\"height\":58,\"top\":46,\"right\":393,\"bottom\":104,\"left\":304.125},\"title\":{\"x\":25,\"y\":748.09375,\"width\":343,\"height\":137,\"top\":748.09375,\"right\":368,\"bottom\":885.09375,\"left\":25},\"subtitle\":{\"x\":25,\"y\":885.09375,\"width\":343,\"height\":84,\"top\":885.09375,\"right\":368,\"bottom\":969.09375,\"left\":25},\"bullet-1\":{\"x\":43,\"y\":979.09375,\"width\":274.21875,\"height\":26,\"top\":979.09375,\"right\":317.21875,\"bottom\":1005.09375,\"left\":43},\"bullet-2\":{\"x\":43,\"y\":1005.09375,\"width\":236.109375,\"height\":26,\"top\":1005.09375,\"right\":279.109375,\"bottom\":1031.09375,\"left\":43},\"bullet-3\":{\"x\":43,\"y\":1031.09375,\"width\":269.71875,\"height\":26,\"top\":1031.09375,\"right\":312.71875,\"bottom\":1057.09375,\"left\":43},\"bullet-4\":{\"x\":43,\"y\":1057.09375,\"width\":238.125,\"height\":26,\"top\":1057.09375,\"right\":281.125,\"bottom\":1083.09375,\"left\":43},\"bullet-5\":{\"x\":43,\"y\":1083.09375,\"width\":195.71875,\"height\":26,\"top\":1083.09375,\"right\":238.71875,\"bottom\":1109.09375,\"left\":43},\"bullet-6\":{\"x\":43,\"y\":1109.09375,\"width\":259.484375,\"height\":26,\"top\":1109.09375,\"right\":302.484375,\"bottom\":1135.09375,\"left\":43},\"timeline title\":{\"x\":25,\"y\":1170.09375,\"width\":343,\"height\":62,\"top\":1170.09375,\"right\":368,\"bottom\":1232.09375,\"left\":25},\"Timeline Row 1 Title\":{\"x\":40,\"y\":1267.09375,\"width\":77.640625,\"height\":32,\"top\":1267.09375,\"right\":117.640625,\"bottom\":1299.09375,\"left\":40},\"Timeline Row 1 Subtitle\":{\"x\":24,\"y\":1299.09375,\"width\":344,\"height\":75,\"top\":1299.09375,\"right\":368,\"bottom\":1374.09375,\"left\":24},\"Timeline Row 2 Title\":{\"x\":40,\"y\":1401.09375,\"width\":117.265625,\"height\":32,\"top\":1401.09375,\"right\":157.265625,\"bottom\":1433.09375,\"left\":40},\"Timeline Row 2 Subtitle\":{\"x\":24,\"y\":1433.09375,\"width\":344,\"height\":75,\"top\":1433.09375,\"right\":368,\"bottom\":1508.09375,\"left\":24},\"Timeline Row 3 Title\":{\"x\":40,\"y\":1535.09375,\"width\":140.484375,\"height\":32,\"top\":1535.09375,\"right\":180.484375,\"bottom\":1567.09375,\"left\":40},\"Timeline Row 3 Subtitle\":{\"x\":24,\"y\":1567.09375,\"width\":344,\"height\":50,\"top\":1567.09375,\"right\":368,\"bottom\":1617.09375,\"left\":24},\"callout-badge\":{\"x\":279,\"y\":1660.09375,\"width\":90,\"height\":31,\"top\":1660.09375,\"right\":369,\"bottom\":1691.09375,\"left\":279},\"callout title\":{\"x\":30,\"y\":1691.09375,\"width\":333,\"height\":32,\"top\":1691.09375,\"right\":363,\"bottom\":1723.09375,\"left\":30},\"callout subtitle\":{\"x\":30,\"y\":1721.09375,\"width\":333,\"height\":25,\"top\":1721.09375,\"right\":363,\"bottom\":1746.09375,\"left\":30},\"Heading 26\":{\"x\":25,\"y\":2277.09375,\"width\":343,\"height\":84,\"top\":2277.09375,\"right\":368,\"bottom\":2361.09375,\"left\":25},\"Privacy Policy\":{\"x\":25,\"y\":2449.09375,\"width\":343,\"height\":44,\"top\":2449.09375,\"right\":368,\"bottom\":2493.09375,\"left\":25},\"Get in touch\":{\"x\":78.0625,\"y\":2576.09375,\"width\":240.859375,\"height\":22,\"top\":2576.09375,\"right\":318.921875,\"bottom\":2598.09375,\"left\":78.0625},\"Already a member\":{\"x\":36,\"y\":2668.09375,\"width\":152.375,\"height\":40,\"top\":2668.09375,\"right\":188.375,\"bottom\":2708.09375,\"left\":36},\"Restore\":{\"x\":284.03125,\"y\":2669.09375,\"width\":82.96875,\"height\":38,\"top\":2669.09375,\"right\":367,\"bottom\":2707.09375,\"left\":284.03125},\"purchase button subtitle\":{\"x\":15,\"y\":783,\"width\":363,\"height\":22,\"top\":783,\"right\":378,\"bottom\":805,\"left\":15},\"purchase-primary\":{\"x\":15,\"y\":713,\"width\":363,\"height\":60,\"top\":713,\"right\":378,\"bottom\":773,\"left\":15},\"custom-1\":{\"x\":139.1875,\"y\":2538.09375,\"width\":109.625,\"height\":38,\"top\":2538.09375,\"right\":248.8125,\"bottom\":2576.09375,\"left\":139.1875}},\"products\":[{\"product\":\"primary\"}],\"fonts\":[{\"family\":\"Rubik, sans-serif\"},{\"family\":\"Lato, sans-serif\"}],\"version\":\"2023-01-12\",\"page_styles\":[{\"to\":\"rgba(0, 0, 0, 1)\",\"from\":\"rgba(0, 0, 0, 1)\",\"property\":\"borderTopColor\"},{\"to\":\"rgba(0, 0, 0, 1)\",\"from\":\"rgba(0, 0, 0, 1)\",\"property\":\"borderBottomColor\"},{\"to\":\"rgba(0, 0, 0, 1)\",\"from\":\"rgba(0, 0, 0, 1)\",\"property\":\"borderRightColor\"},{\"to\":\"rgba(0, 0, 0, 1)\",\"from\":\"rgba(0, 0, 0, 1)\",\"property\":\"borderLeftColor\"},{\"to\":\"rgba(0, 0, 0, 1)\",\"from\":\"rgba(0, 0, 0, 1)\",\"property\":\"color\"},{\"to\":\"rgba(44, 51, 64, 1)\",\"from\":\"rgba(44, 51, 64, 1)\",\"property\":\"borderTopColor\"},{\"to\":\"rgba(44, 51, 64, 1)\",\"from\":\"rgba(44, 51, 64, 1)\",\"property\":\"borderBottomColor\"},{\"to\":\"rgba(44, 51, 64, 1)\",\"from\":\"rgba(44, 51, 64, 1)\",\"property\":\"borderRightColor\"},{\"to\":\"rgba(44, 51, 64, 1)\",\"from\":\"rgba(44, 51, 64, 1)\",\"property\":\"borderLeftColor\"},{\"to\":\"rgba(24, 26, 31, 1)\",\"from\":\"rgba(24, 26, 31, 1)\",\"property\":\"backgroundColor\"},{\"to\":\"rgba(44, 51, 64, 1)\",\"from\":\"rgba(44, 51, 64, 1)\",\"property\":\"color\"},{\"to\":\"rgba(24, 26, 31, 0.9)\",\"from\":\"rgba(24, 26, 31, 0.9)\",\"property\":\"backgroundColor\"},{\"to\":\"rgba(255, 255, 255, 0.47)\",\"from\":\"rgba(255, 255, 255, 0.47)\",\"property\":\"borderTopColor\"},{\"to\":\"rgba(255, 255, 255, 0.47)\",\"from\":\"rgba(255, 255, 255, 0.47)\",\"property\":\"borderBottomColor\"},{\"to\":\"rgba(255, 255, 255, 0.47)\",\"from\":\"rgba(255, 255, 255, 0.47)\",\"property\":\"borderRightColor\"},{\"to\":\"rgba(255, 255, 255, 0.47)\",\"from\":\"rgba(255, 255, 255, 0.47)\",\"property\":\"borderLeftColor\"},{\"to\":\"rgba(255, 255, 255, 0.47)\",\"from\":\"rgba(255, 255, 255, 0.47)\",\"property\":\"color\"},{\"to\":\"rgba(255, 255, 255, 1)\",\"from\":\"rgba(255, 255, 255, 1)\",\"property\":\"borderTopColor\"},{\"to\":\"rgba(255, 255, 255, 1)\",\"from\":\"rgba(255, 255, 255, 1)\",\"property\":\"borderBottomColor\"},{\"to\":\"rgba(255, 255, 255, 1)\",\"from\":\"rgba(255, 255, 255, 1)\",\"property\":\"borderRightColor\"},{\"to\":\"rgba(255, 255, 255, 1)\",\"from\":\"rgba(255, 255, 255, 1)\",\"property\":\"borderLeftColor\"},{\"to\":\"rgba(255, 255, 255, 1)\",\"from\":\"rgba(255, 255, 255, 1)\",\"property\":\"color\"},{\"to\":\"rgba(73, 144, 225, 0.13)\",\"from\":\"rgba(73, 144, 225, 0.13)\",\"property\":\"backgroundColor\"},{\"to\":\"rgba(73, 158, 255, 1)\",\"from\":\"rgba(73, 158, 255, 1)\",\"property\":\"backgroundColor\"},{\"to\":\"rgba(255, 255, 255, 0.51)\",\"from\":\"rgba(255, 255, 255, 0.51)\",\"property\":\"borderTopColor\"},{\"to\":\"rgba(255, 255, 255, 0.51)\",\"from\":\"rgba(255, 255, 255, 0.51)\",\"property\":\"borderBottomColor\"},{\"to\":\"rgba(255, 255, 255, 0.51)\",\"from\":\"rgba(255, 255, 255, 0.51)\",\"property\":\"borderRightColor\"},{\"to\":\"rgba(255, 255, 255, 0.51)\",\"from\":\"rgba(255, 255, 255, 0.51)\",\"property\":\"borderLeftColor\"},{\"to\":\"rgba(255, 255, 255, 0.51)\",\"from\":\"rgba(255, 255, 255, 0.51)\",\"property\":\"color\"},{\"to\":\"rgba(255, 255, 255, 0.05)\",\"from\":\"rgba(255, 255, 255, 0.05)\",\"property\":\"backgroundColor\"},{\"to\":\"rgba(255, 255, 255, 0.07)\",\"from\":\"rgba(255, 255, 255, 0.07)\",\"property\":\"backgroundColor\"},{\"to\":\"rgba(255, 255, 255, 0.17)\",\"from\":\"rgba(255, 255, 255, 0.17)\",\"property\":\"borderTopColor\"},{\"to\":\"rgba(0, 0, 0, 0.49) 0px -1px 32px 6px\",\"from\":\"rgba(0, 0, 0, 0.49) 0px -1px 32px 6px\",\"property\":\"boxShadow\"}]}]}}
"""
    let message = MockWKScriptMessage(body: body)

    rawWebMessageHandler.userContentController(
      WKUserContentController(),
      didReceive: message
    )

    XCTAssertEqual(delegate.handledMessages, [.onReady(paywallJsVersion: "2023-01-12")])
  }
}
